use Test::More;

use_ok 'Net::OpenNebula';
use_ok 'Net::OpenNebula::VM';
use_ok 'Net::OpenNebula::Template';
use_ok 'Net::OpenNebula::Host';
use_ok 'Net::OpenNebula::Cluster';
use_ok 'Net::OpenNebula::VM::NIC';

done_testing();
